/* eslint-disable no-template-curly-in-string */
const chalk = require('chalk');
const packagejs = require('../../package.json');
const semver = require('semver');
const BaseGenerator = require('generator-jhipster/generators/generator-base');
const jhipsterConstants = require('generator-jhipster/generators/generator-constants');

module.exports = class extends BaseGenerator {
    get initializing() {
        return {
            readConfig() {
                this.jhipsterAppConfig = this.getJhipsterAppConfig();
                if (!this.jhipsterAppConfig) {
                    this.error('Can\'t read .yo-rc.json');
                }
            },
            displayLogo() {
                // it's here to show that you can use functions from generator-jhipster
                // this function is in: generator-jhipster/generators/generator-base.js
                this.printJHipsterLogo();

                // Have Yeoman greet the user.
                this.log(`\nWelcome to the ${chalk.bold.yellow('JHipster elasticsearch-rest-client')} generator! ${chalk.yellow(`v${packagejs.version}\n`)}`);
            },
            checkJhipster() {
                const currentJhipsterVersion = this.jhipsterAppConfig.jhipsterVersion;
                const minimumJhipsterVersion = packagejs.dependencies['generator-jhipster'];
                if (!semver.satisfies(currentJhipsterVersion, minimumJhipsterVersion)) {
                    this.warning(`\nYour generated project used an old JHipster version (${currentJhipsterVersion})... you need at least (${minimumJhipsterVersion})\n`);
                }
            }
        };
    }

    prompting() {
        const prompts = [
            {
                type: 'input',
                name: 'message',
                message: 'Are you add elasticsearch-rest-client for you project? <yes/no>',
                default: 'yes'
            }
        ];

        const done = this.async();
        this.prompt(prompts).then((props) => {
            this.props = props;
            // To access props later use this.props.someOption;
            this.message = this.props.message;
            done();
        });
    }

    writing() {
        // read config from .yo-rc.json
        this.baseName = this.jhipsterAppConfig.baseName;
        this.packageName = this.jhipsterAppConfig.packageName;
        this.packageFolder = this.jhipsterAppConfig.packageFolder;
        this.clientFramework = this.jhipsterAppConfig.clientFramework;
        this.clientPackageManager = this.jhipsterAppConfig.clientPackageManager;
        this.buildTool = this.jhipsterAppConfig.buildTool;

        // use function in generator-base.js from generator-jhipster
        this.angularAppName = this.getAngularAppName();

        // use constants from generator-constants.js
        const javaDir = `${jhipsterConstants.SERVER_MAIN_SRC_DIR + this.packageFolder}/`;
        // variable from questions
        if (this.message === 'yes') {
            const buildGradle = 'build.gradle';
            const buildGradleSource = 'compile "org.springframework.boot:spring-boot-starter-data-elasticsearch"';
            const buildGradleTarget = 'compile "org.elasticsearch.client:elasticsearch-rest-client:${elasticsearch_rest_client_version}"';

            const buildProperties = 'gradle.properties';
            const buildPropertiesSource = '# Overridden to get metrics-jcache';
            const buildPropertiesTarget = 'elasticsearch_rest_client_version=5.6.3';

            const constants = `${javaDir}/config/Constants.java`;
            const constantsSource = 'public static final String ANONYMOUS_USER = "anonymoususer";';
            const constantsTarget = 'public static final String ELASTICSEARCH_ALL_INDEX = "_all";';

            const config = `${javaDir}/config/ElasticsearchConfiguration.java`;
            const configSourceHost = 'public class ElasticsearchConfiguration {';
            const configTargetHost = '\n\t@Value("${spring.data.elasticsearch.rest-client}")\n\tprivate String restClientHost;\n\n' +
                '\t@Bean\n' +
                '\tpublic RestClient restElasticsearchClient() {\n' +
                '\t\treturn RestClient.builder(\n' +
                '\t\t\tHttpHost.create(this.restClientHost)\n' +
                '\t\t).build();\n' +
                '\t}';

            const configImportSource = 'import com.fasterxml.jackson.databind.ObjectMapper;';
            const configImportTarget = 'import org.apache.http.HttpHost;\n' +
                'import org.springframework.beans.factory.annotation.Value;\n' +
                'import org.elasticsearch.client.RestClient;';
            if (!this.fs.read(buildGradle, {
                defaults: ''
            }).includes(buildGradleTarget)) {
                // 添加gradle
                this.replaceContent(
                    buildGradle, buildGradleSource,
                    `${buildGradleSource}\n\t${buildGradleTarget}`
                );

                // 添加properties
                this.replaceContent(
                    buildProperties, buildPropertiesSource,
                    `${buildPropertiesSource}\n${buildPropertiesTarget}`
                );

                // 添加Contant
                this.replaceContent(
                    constants, constantsSource,
                    `${constantsSource}\n\t${constantsTarget}`
                );

                // 添加config
                this.replaceContent(
                    config, configSourceHost,
                    `${configSourceHost}\n${configTargetHost}`
                );
                this.replaceContent(
                    config, configImportSource,
                    `${configImportSource}\n${configImportTarget}`
                );

                // 复制文件
                this.fs.copyTpl(
                    this.templatePath('_ElasticsearchUtil.java'),
                    this.destinationPath(`${javaDir}/service/util/ElasticsearchUtil.java`), this
                );
            }
        }
    }

    install() {
    }

    end() {
        this.log('End of elasticsearch-rest-client generator');
    }
};
